#include <MediaDefs.h>
#include <Buffer.h>
#include <BufferGroup.h>
#include <MediaRoster.h>
#include <TimeSource.h>
#include <Application.h>

#include <stdio.h>
#include <stdlib.h>

#include "VUMeterNode.h"
#include "VUMeterView.h"

#if B_HOST_IS_BENDIAN
#define ENDIAN B_MEDIA_BIG_ENDIAN
#else
#define ENDIAN B_MEDIA_LITTLE_ENDIAN
#endif

//#define DEBUG 
#ifdef DEBUG
	#define PRINT printf
#else
	static inline void PRINT( const char*, ... ) {}
#endif

VUMeterNode::VUMeterNode()
:	BMediaNode("VUMeterNode"),
	BBufferConsumer(B_MEDIA_RAW_AUDIO),
	BMediaEventLooper()
{
	PRINT( "VUMeterNode\n" );

	pWindow = new BWindow( 
		BRect(100,100,620,240), 
		"VUMeter", B_TITLED_WINDOW, 
		B_NOT_ZOOMABLE | B_NOT_RESIZABLE );

	pLeftView = new VUMeterView(BRect(0,0,260,140));
	pRightView = new VUMeterView(BRect(260,0,520,140));
	pWindow->AddChild( pLeftView );
	pWindow->AddChild( pRightView );

	pWindow->Show();
}

VUMeterNode::~VUMeterNode()
{
	PRINT( "~VUMeterNode\n" );
	
	Quit();

	if ( pWindow ) {
		if ( pWindow->Lock() ) {
			pWindow->Close();
			pWindow = NULL;
		}
	}
}

BMediaAddOn* 
VUMeterNode::AddOn(int32 *internal_id ) const
{
	PRINT( "AddOn\n" );
	
	return NULL;
}

status_t 
VUMeterNode::GetNextInput(
		int32 *cookie,
		media_input *out_input)
{
	PRINT( "GetNextInput\n" );

	if (*cookie == 0) {
		mInput.format.type = B_MEDIA_RAW_AUDIO;
		mInput.format.u.raw_audio = media_raw_audio_format::wildcard;
		mInput.node = Node();
		mInput.destination.port = ControlPort();
		mInput.destination.id = 1;
		strcpy( mInput.name, Name());

		*out_input = mInput;
		*cookie = 1;
		return B_OK;
	}

	return B_BAD_INDEX;
}

void 
VUMeterNode::DisposeInputCookie(
		int32 cookie)
{
	PRINT( "DisposeInputCookie\n" );
}

status_t 
VUMeterNode::AcceptFormat(
		const media_destination &destination,
		media_format *format)
{
	PRINT( "AcceptFormat\n" );

	if ( format->type != B_MEDIA_RAW_AUDIO )
		return B_MEDIA_BAD_FORMAT;

	if ( format->u.raw_audio.format 
				== media_raw_audio_format::wildcard.format 
			|| format->u.raw_audio.format 
				== media_raw_audio_format::B_AUDIO_SHORT )
		format->u.raw_audio.format = media_raw_audio_format::B_AUDIO_SHORT;
	else
		return B_MEDIA_BAD_FORMAT;
	
	if ( format->u.raw_audio.channel_count 
				== media_raw_audio_format::wildcard.channel_count 
			|| format->u.raw_audio.channel_count == 2 )
		format->u.raw_audio.channel_count = 2;
	else 
		return B_MEDIA_BAD_FORMAT;

	if ( format->u.raw_audio.frame_rate 
			== media_raw_audio_format::wildcard.frame_rate )
		format->u.raw_audio.frame_rate = 44100;

	if ( format->u.raw_audio.byte_order 
			== media_raw_audio_format::wildcard.byte_order )
		format->u.raw_audio.byte_order = ENDIAN;

	if ( format->u.raw_audio.buffer_size 
			== media_raw_audio_format::wildcard.buffer_size )
		format->u.raw_audio.buffer_size = 1024;

	mInput.format = *format;

	return B_OK;
}

status_t 
VUMeterNode::Connected(
		const media_source &source,
		const media_destination &destination,
		const media_format &format,
		media_input *out_input)
{
	PRINT( "Connected\n" );

	if ( destination != mInput.destination ) 
		return B_MEDIA_BAD_DESTINATION;

	mInput.source = source;
	mInput.format = format;
	*out_input = mInput;
	
	return B_OK;
}

void 
VUMeterNode::Disconnected(
		const media_source &source,
		const media_destination &destination )
{
	PRINT( "Disconnected\n" );
}

status_t 
VUMeterNode::GetLatencyFor(
		const media_destination &destination,
		bigtime_t *out_latency,
		media_node_id *out_timesource)
{
	PRINT( "GetLatencyFor\n" );

	status_t err = 0;
	
	*out_latency = 50 * 1000;
	*out_timesource = TimeSource()->ID();
	
	return err;
}

void 
VUMeterNode::BufferReceived( 
		BBuffer *buffer)
{
	PRINT( "BufferReceived\n" );

	status_t err;
	
	media_timed_event event(
		buffer->Header()->start_time, 
		BTimedEventQueue::B_HANDLE_BUFFER,
		buffer, 
		BTimedEventQueue::B_RECYCLE_BUFFER);
		
	err = EventQueue()->AddEvent(event);
	if (err) 
		buffer->Recycle();
}

void 
VUMeterNode::ProducerDataStatus(
		const media_destination &destination,
		int32 status,
		bigtime_t at_performance_time)
{
	PRINT( "ProducerDataStatus\n" );
}

status_t 
VUMeterNode::FormatChanged(
		const media_source &source,
		const media_destination &destination, 
		int32 change_tag,
		const media_format &format)
{
	PRINT( "FormatChanged\n" );

	status_t err = 0;
	
	return err;
}

void 
VUMeterNode::HandleEvent(
		const media_timed_event *event,
		bigtime_t lateness,
		bool realTimeEvent)
{
	PRINT( "HandleEvent\n" );

	BBuffer* buffer;
	
	switch (event->type) {

	case BTimedEventQueue::B_HANDLE_BUFFER:
		buffer = const_cast<BBuffer*>((BBuffer*) event->pointer);
		if ( !buffer )
			break;			
		Measure( buffer );
		buffer->Recycle();
		break;
	
	default:
		break;
	}
}

void VUMeterNode::Measure( BBuffer* buffer )
{
	double l = 0.0, r = 0.0, t, u;
	size_t i, frame_count;

	media_raw_audio_format *format = &mInput.format.u.raw_audio;

	if ( format->format == media_raw_audio_format::B_AUDIO_SHORT ) {	
		short* data = (short*)buffer->Data();
		frame_count = buffer->SizeUsed() / sizeof(short) / format->channel_count;

		if ( format->byte_order != ENDIAN ) {
			for ( i=0; i < frame_count; i++) {
				t = short( B_SWAP_INT16( data[i * 2] ) ) / 32768.0;
				u = short( B_SWAP_INT16( data[i * 2 + 1] ) ) / 32768.0;
				l += t * t;
				r += u * u;
			} 
		} else {
			for ( i=0; i < frame_count; i++) {
				t = double(data[i * 2]) / 32768.0;
				u = double(data[i * 2 + 1]) / 32768.0;
				l += t * t;
				r += u * u;
			}
		}
	} else 
		return;
	
	// average power
	l /= double(frame_count);
	r /= double(frame_count);
	PRINT( "avp L: %f | R: %f \n", l, r );

	// r.m.s.
	l = sqrt(l);
	r = sqrt(r);
	PRINT( "rms L: %f | R: %f \n", l, r );

	// dB ( -inf to 0 )
	l = log10(l) * 20.0;
	r = log10(r) * 20.0;
	PRINT( " dB L: %f | R: %f \n", l, r );

	BMessage* msg;
	msg = new BMessage( VALUE_DECIBEL_MSG );
	msg->AddDouble( "value", l );
	pWindow->PostMessage( msg, pLeftView );
	delete msg;

	msg = new BMessage( VALUE_DECIBEL_MSG );
	msg->AddDouble( "value", r );
	pWindow->PostMessage( msg, pRightView );
	delete msg;
}

