#include <Window.h>
#include <stdio.h>
#include <math.h>

#include "VUMeterView.h"

// This view assumes its size be 260 x 160

#define RANGE_DECIBEL 60.0
#define NEEDLE_RESPONSE 1.3
#define Y_OFFSET 30
#define R_FACTOR 0.5

#define NEEDLE_TOP 140
#define NEEDLE_BOTTOM 50
#define SCALE_TOP 155
#define SCALE_BOTTOM 150

const double scale[] = { 0, -6, -12, -24, -40, -60 };
const int number_of_scale = 6;

VUMeterView::VUMeterView( BRect frame )
:	BView( frame, "VUMeterView", B_FOLLOW_ALL_SIDES, 
			B_WILL_DRAW | B_PULSE_NEEDED ),
	mDisplayValue(0.0),
	mCurrentValue(0.0),
	mCounter(0)
{
	SetViewColor( 0xff, 0xff, 0xff);
	SetHighColor( 0x00, 0x00, 0x00);
	mNeedleTop.Set( 0.0, 0.0 );
	mNeedleBottom.Set( 0.0, 0.0 );
}

VUMeterView::~VUMeterView()
{
}

void VUMeterView::AttachedToWindow()
{
	Window()->SetPulseRate( 30 * 1000 );
}

void VUMeterView::Draw( BRect updateRect )
{
	DrawString( "VU", BPoint( 30, 130) );
	StrokeLine( mNeedleTop, mNeedleBottom );
	
	for ( int i = 0; i < number_of_scale; i++ )
		DrawScale( scale[i], SCALE_TOP, SCALE_BOTTOM );
}

void VUMeterView::DrawScale( double db, double top, double bottom )
{
	BPoint a, b;
	double radian, x, y, value;
	value = db / RANGE_DECIBEL + 1.0;
	if ( value < 0.0 ) value = 0.0;

	radian = PI * ( value * R_FACTOR + R_FACTOR / 2.0);
	x = cos(radian);
	y = sin(radian);
	
	BRect r = Frame();
	a.x = r.Width() / 2 - x * top;
	a.y = r.Height() + Y_OFFSET - y * top;
	b.x = r.Width() / 2 - x * bottom;
	b.y = r.Height() + Y_OFFSET - y * bottom;

	StrokeLine( a, b );

	char s[16];
	sprintf( s, "%d", int(db) );
	a += BPoint( 3, 0 );
	DrawString( s, a );	
}

void VUMeterView::MessageReceived( BMessage* message )
{
	double value;
	switch ( message->what ) {
	case VALUE_DECIBEL_MSG:
		value = message->FindDouble( "value" );

		// 0.0 to 1.0 ( mapping from -RANGE_DECIBEL dB to 0dB )
		value = value / RANGE_DECIBEL + 1.0;
		if ( value < 0.0 ) value = 0.0;

		mCurrentValue = value;
		mCounter = 0;
		break;

	default:
		BView::MessageReceived( message );
	}
}

void VUMeterView::Pulse()
{
	double delta, radian, x, y;
	rgb_color rgb;
	
	if ( mCounter++ > 10 ) {
		mCurrentValue = 0;
	}
		
	delta = (mCurrentValue - mDisplayValue) / NEEDLE_RESPONSE;
	mDisplayValue += delta;
	radian = PI * (mDisplayValue * R_FACTOR + R_FACTOR / 2.0);
	x = cos(radian);
	y = sin(radian);

	if ( Window()->Lock() ) {
		rgb = HighColor();
		SetHighColor( ViewColor() );
		StrokeLine( mNeedleTop, mNeedleBottom );

		BRect r = Frame();
		mNeedleTop.x = r.Width() / 2 - x * NEEDLE_TOP;
		mNeedleTop.y = r.Height() + Y_OFFSET - y * NEEDLE_TOP;
		mNeedleBottom.x = r.Width() / 2 - x * NEEDLE_BOTTOM;
		mNeedleBottom.y = r.Height() + Y_OFFSET - y * NEEDLE_BOTTOM;

		SetHighColor( rgb );
		StrokeLine( mNeedleTop, mNeedleBottom );
		Window()->Unlock();
	}
}

