#include <Application.h>
#include <MediaRoster.h>
#include <MediaAddOn.h>
#include <MediaDefs.h>
#include <TimeSource.h>
#include <Entry.h>
#include <Path.h>

#include <stdio.h>
#include <stdlib.h>

#include "VUMeterNode.h"

#define ERROR( err, msg ) \
	{ if ( err != B_OK ) {		\
		printf( "error: %s - %s\n", msg, strerror(err) );	\
		return -1; } }

// Test main() for app based node.

int main(int argc, char** argv)
{
	status_t err;
	int32 count;
	char *filename;	
	BApplication app("application/x-vnd.vumeter_test");

	if (argc != 2) {
		printf("#Usage: %s <filename>\n", argv[0]);
		filename = "/boot/optional/sound/Startup Sounds/12StrStart.aiff";
		printf("Temp file, %s\n", filename);
//		return -1;
	} else
		filename = argv[1];
	
	BMediaRoster* roster = BMediaRoster::Roster(&err);	
	if ( roster == NULL || err != B_OK ) {
		printf("BMediaRoster::Roster() error. - %s\n", strerror(err));
		return -1;
	}

//	Get fileplayer
	media_node fileplayer;
	entry_ref ref;
	dormant_node_info dinfo;

	err = get_ref_for_path( filename, &ref );
	ERROR( err, "get_ref_for_path()");

	err = roster->SniffRef( ref, B_BUFFER_PRODUCER, &dinfo );
	ERROR( err, "BMediaRoster::SniffRef()");
	printf("Dormant node: %s\n", dinfo.name );

	err = roster->InstantiateDormantNode( dinfo, &fileplayer );
	ERROR( err, "BMediaRoster::InstantiateDormantNode()");

	BMediaNode* mynode;

	bigtime_t duration;
	err = roster->SetRefFor( fileplayer, ref, false, &duration );
	ERROR( err, "BMediaRoster::SetRefFor()");

//	Get testing node
	media_node vumeter;
	mynode = new VUMeterNode();
	err = roster->RegisterNode( mynode );
	roster->GetNodeFor( mynode->Node().node, &vumeter );

//	Get end points
	media_input vumeterInput;
	media_output soundOutput;
	count = 1;

	err = roster->GetFreeOutputsFor( fileplayer, &soundOutput, 1, &count);
	ERROR( err, "BMediaRoster::GetFreeOutputsFor()");

	count = 1;
	err = roster->GetFreeInputsFor( vumeter, &vumeterInput, 1, &count);
	ERROR( err, "BMediaRoster::GetFreeInputsFor()");

//	Try connection
	media_format format;
	format.type = B_MEDIA_RAW_AUDIO;	
	format.u.raw_audio = media_raw_audio_format::wildcard;
	err = roster->Connect(
			soundOutput.source, 
			vumeterInput.destination, 
			&format, 
			&soundOutput, 
			&vumeterInput);
	ERROR( err, "BMediaRoster::Connect()");

//	Set time source
	media_node timeSource;
	err = roster->GetTimeSource( &timeSource );
	ERROR( err, "BMediaRoster::GetTimeSource()");

	err = roster->SetTimeSourceFor( fileplayer.node, timeSource.node );
	ERROR( err, "BMediaRoster::SetTimeSourceFor()");
	err = roster->SetTimeSourceFor( vumeter.node, timeSource.node );
	ERROR( err, "BMediaRoster::SetTimeSourceFor()");

//	Now let's play it !
	bigtime_t startTime;
	err = roster->GetStartLatencyFor( timeSource, &startTime );
	ERROR( err, "BMediaRoster::GetStartLatencyFor(timeSource)");	
	printf("start latency: %Ld\n", startTime ); 

	BTimeSource *bTimeSource;
	bTimeSource = roster->MakeTimeSourceFor( timeSource );
	startTime += bTimeSource->PerformanceTimeFor( BTimeSource::RealTime() 
		+ 100 * 1000 );

	err = roster->StartNode( fileplayer, startTime );
	ERROR( err, "BMediaRoster::StartNode(fileplayer)");
	err = roster->StartNode( vumeter, startTime );
	ERROR( err, "BMediaRoster::StartNode(fileplayer)");

	snooze( 10*1000*1000 );

	err = roster->StopNode( fileplayer, 0, true );
	ERROR( err, "BMediaRoster::StopNode(fileplayer)");
	err = roster->StopNode( vumeter, 0, true );
	ERROR( err, "BMediaRoster::StopNode(fileplayer)");

//	Clean-up

//	This is work-around for a bug on Be media reader 4.5.0
//	Usually you should do Disconnect.
/*	err = roster->Disconnect(
			fileplayer.node, soundOutput.source, 
			mixer.node, mixerInput.destination );
	ERROR( err, "BMediaRoster::Disconnect()");
*/
	err = roster->ReleaseNode( fileplayer );
	ERROR( err, "BMediaRoster::ReleaseNode(fileplayer)");
	err = roster->ReleaseNode( vumeter );
	ERROR( err, "BMediaRoster::ReleaseNode(vumeter)");

	err = roster->UnregisterNode( mynode );
	ERROR( err, "BMediaRoster::UnregisterNode(mynode)");

	bTimeSource->Release();		
//	Never do this.
//	err = roster->ReleaseNode( timeSource );

	printf( "\n" );
	return 0;
}

